package com.aidlab.aidlab_sdk

import com.aidlab.sdk.communication.*
import com.aidlab.sdk.internal.InternalDelegate
import com.aidlab.sdk.internal.InternalUpdateDelegate
import io.flutter.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin

class AidlabHandler(
        val synchronizationDelegate: AidlabSynchronizationDelegate
): AidlabDelegate, InternalDelegate, InternalUpdateDelegate, AidlabFlutter.PlatformAidlab {

    private var released = false

    private var aidlabListener: AidlabFlutter.PlatformAidlabDelegate? = null
    private var internalListener: AidlabFlutter.PlatformAidlabInternalDelegate? = null
    private var updateListener: AidlabFlutter.PlatformAidlabUpdateDelegate? = null

    private var connectedAidlab: IAidlab? = null
    private var isConnected = false

    fun init(binding: FlutterPlugin.FlutterPluginBinding) {
        released = false
        aidlabListener = AidlabFlutter.PlatformAidlabDelegate(binding.binaryMessenger)
        internalListener = AidlabFlutter.PlatformAidlabInternalDelegate(binding.binaryMessenger)
        updateListener = AidlabFlutter.PlatformAidlabUpdateDelegate(binding.binaryMessenger)
        AidlabFlutter.PlatformAidlab.setup(binding.binaryMessenger, this)
    }
    
    fun release() {
        released = true
        connectedAidlab?.disconnect()
        aidlabListener = null
    }

    override fun disconnect() {
        connectedAidlab?.disconnect()
    }

    override fun send(arg: AidlabFlutter.SendRequest?) {
        connectedAidlab?.send(arg!!.command)
    }

    override fun setECGFiltrationMethod(arg: AidlabFlutter.EcgFiltrationRequest?) {
        connectedAidlab?.setECGFiltrationMethod(ECGFiltrationMethod.values()[arg!!.method.toInt()])
    }

    override fun startSynchronization() {
        connectedAidlab?.startSynchronization()
    }

    override fun stopSynchronization() {
        connectedAidlab?.stopSynchronization()
    }

    override fun requestForFirmwareInfo(arg: AidlabFlutter.FirmwareInfoRequest?) {
        connectedAidlab?.internalAidlab?.requestForFirmwareInfo(arg!!.url)
    }

    override fun startFirmwareUpdate(arg: AidlabFlutter.FirmwareUpdateRequest?) {
        connectedAidlab?.internalAidlab?.startFirmwareUpdate(arg!!.url, arg.updatableVersion)
    }

    override fun didConnectAidlab(aidlab: IAidlab) {
        if(released) {
            aidlab.disconnect()
            return
        }
        isConnected = true
        connectedAidlab = aidlab
        aidlab.internalAidlab.internalDelegate = this
        aidlab.internalAidlab.updateDelegate = this

        connectedAidlab?.aidlabSynchronizationDelegate = synchronizationDelegate
        aidlabListener?.didConnect(AidlabFlutter.ConnectEvent().apply {
            this.firmwareRevision = aidlab.firmwareRevision
            this.hardwareRevision = aidlab.hardwareRevision
            this.serialNumber = aidlab.serialNumber
        }) {}
    }

    override fun didDisconnectAidlab(aidlab: IAidlab) {
        isConnected = false

        connectedAidlab?.aidlabSynchronizationDelegate = null
        connectedAidlab = null
        aidlabListener?.didDisconnect(AidlabFlutter.EmptyEvent()) {}
    }

    override fun didReceiveECG(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        aidlabListener?.didReceiveEcg(AidlabFlutter.EcgEvent().apply {
            this.timestamp = timestamp
            this.values = DoubleArray(values.size) { values[it].toDouble() }
        }) {}
    }

    override fun didDetectExercise(aidlab: IAidlab, exercise: Exercise) {
        aidlabListener?.didDetectExercise(AidlabFlutter.ExerciseEvent().apply {
            this.exercise = exercise.value.toLong()
        }) {}
    }

    override fun didReceiveAccelerometer(aidlab: IAidlab, timestamp: Long, ax: Float, ay: Float, az: Float) {
        aidlabListener?.didReceiveAccelerometer(Utils.createVectorEvent(
                timestamp, ax, ay, az
        )) {}
    }

    override fun didReceiveActivity(aidlab: IAidlab, timestamp: Long, activity: ActivityType) {
        aidlabListener?.didReceiveActivity(AidlabFlutter.ActivityEvent().apply {
            this.timestamp = timestamp
            this.activityType = activity.value.toLong()
        }) {}
    }

    override fun didReceiveBatteryLevel(aidlab: IAidlab, stateOfCharge: Int) {
        aidlabListener?.didReceiveBatteryLevel(AidlabFlutter.BatteryLevelEvent().apply {
            this.stateOfCharge = stateOfCharge.toLong()
        }) {}
    }

    override fun didReceiveCommand(aidlab: IAidlab) {
        Log.d("AidlabHandler", "didReceiveCommand")
        aidlabListener?.didReceiveCommand(AidlabFlutter.EmptyEvent()) {
            Log.d("AidlabHandler", "didReceiveCommand on respose")
        }
    }

    override fun didReceiveError(error: String) {
        aidlabListener?.didReceiveError(AidlabFlutter.ErrorEvent().apply {
            this.error = error
        }) {}
    }

    override fun didReceiveFirmwareInformation(updateAvaible: Boolean, updatableVersion: String) {
        internalListener?.didReceiveFirmwareInformation(AidlabFlutter.FirmwareInformationEvent().apply {
            this.updateAvailable = updateAvailable
            this.updatableVersion = updatableVersion
        }) {}
    }

    override fun didReceiveGyroscope(aidlab: IAidlab, timestamp: Long, qx: Float, qy: Float, qz: Float) {
        aidlabListener?.didReceiveGyroscope(Utils.createVectorEvent(
                timestamp, qx, qz, qy
        )) {}
    }

    override fun didReceiveHeartRate(aidlab: IAidlab, timestamp: Long, hrv: IntArray, heartRate: Int) {
        aidlabListener?.didReceiveHeartRate(AidlabFlutter.HeartRateEvent().apply {
            this.timestamp = timestamp
            this.heartRate = heartRate.toLong()
            this.hrv = hrv
        }) {}
    }

    override fun didReceiveMagnetometer(aidlab: IAidlab, timestamp: Long, mx: Float, my: Float, mz: Float) {
        aidlabListener?.didReceiveMagnetometer(Utils.createVectorEvent(
                timestamp, mx, my, mz
        )) {}
    }

    override fun didReceiveMessage(aidlab: IAidlab, process: String, message: String) {
        aidlabListener?.didReceiveMessage(AidlabFlutter.MessageEvent().apply {
            this.process = process
            this.message = message
        }) {}
    }

    override fun didReceiveOrientation(aidlab: IAidlab, timestamp: Long,
                                       roll: Float, pitch: Float, yaw: Float,
                                       bodyPosition: BodyPosition) {
        aidlabListener?.didReceiveOrientation(AidlabFlutter.OrientationEvent().apply {
            this.timestamp = timestamp
            this.roll = roll.toDouble()
            this.pitch = pitch.toDouble()
            this.yaw = yaw.toDouble()
            this.bodyPosition = bodyPosition.value.toLong()
        }) {}
    }

    override fun didReceivePressure(aidlab: IAidlab, timestamp: Long, values: IntArray) {
        internalListener?.didReceivePressure(AidlabFlutter.PressureEvent().apply {
            this.timestamp = timestamp
            this.values = values
        }) {}
    }

    override fun didReceiveQuaternion(aidlab: IAidlab, timestamp: Long, qw: Float, qx: Float, qy: Float, qz: Float) {
        aidlabListener?.didReceiveQuaternion(AidlabFlutter.QuaternionEvent().apply {
            this.timestamp = timestamp
            this.qw = qw.toDouble()
            this.qx = qx.toDouble()
            this.qy = qy.toDouble()
            this.qz = qz.toDouble()
        }) {}
    }

    override fun didReceiveRespiration(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        aidlabListener?.didReceiveRespiration(AidlabFlutter.RespirationEvent().apply {
            this.timestamp = timestamp
            this.values = DoubleArray(values.size) {index -> values[index].toDouble()}
        }) {}
    }

    override fun didReceiveRespirationRate(aidlab: IAidlab, timestamp: Long, value: Int) {
        aidlabListener?.didReceiveRespirationRate(AidlabFlutter.RespirationRateEvent().apply {
            this.timestamp = timestamp
            this.value = value.toLong()
        }) {}
    }

    override fun didReceiveSkinTemperature(aidlab: IAidlab, timestamp: Long, value: Float) {
        aidlabListener?.didReceiveSkinTemperature(AidlabFlutter.SkinTemperatureEvent().apply { 
            this.timestamp = timestamp
            this.value = value.toDouble()
        }) {}
    }

    override fun didReceiveSoundVolume(aidlab: IAidlab, timestamp: Long, value: Int) {
        aidlabListener?.didReceiveSoundVolume(AidlabFlutter.SoundVolumeEvent().apply {
            this.timestamp = timestamp
            this.soundVolume = value.toLong()
        }) {}
    }

    override fun didReceiveSteps(aidlab: IAidlab, timestamp: Long, value: Long) {
        aidlabListener?.didReceiveSteps(AidlabFlutter.StepsEvent().apply {
            this.timestamp = timestamp
            this.value = value
        }) {}
    }

    override fun pressureWearStateDidChange(aidlab: IAidlab, state: WearState) {
        internalListener?.pressureWearStateDidChange(AidlabFlutter.PressureWearStateEvent().apply {
            this.wearState = state.value.toLong()
        }) {}
    }

    override fun wearStateDidChange(aidlab: IAidlab, wearState: WearState) { 
        aidlabListener?.wearStateDidChange(AidlabFlutter.WearStateEvent().apply { 
            this.wearState = wearState.value.toLong()
        }) {}
    }

    override fun didReceiveSoundFeatures(aidlab: IAidlab, values: FloatArray): Unit {

    }

    override fun didDetectUserEvent() {
        internalListener?.didReceiveUserEvent(AidlabFlutter.EmptyEvent()) {}
    }

    override fun didFinishUpdate() {
        updateListener?.didFinishUpdate(AidlabFlutter.EmptyEvent()) {}
    }

    override fun didReceiveUpdateError(error: UpdateError) {
        updateListener?.didReceiveUpdateError(AidlabFlutter.EmptyEvent()) {}
    }

    override fun didReceiveUpdateProgress(progress: Int) {
        updateListener?.didReceiveUpdateProgress(AidlabFlutter.UpdateProgress().apply {
            this.progress = progress.toLong()
        }) {}
    }


}