package com.aidlab.aidlab_sdk

import com.aidlab.sdk.communication.ActivityType
import com.aidlab.sdk.communication.AidlabSynchronizationDelegate
import com.aidlab.sdk.communication.IAidlab
import com.aidlab.sdk.communication.SyncState
import io.flutter.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin

class AidlabSynchronizationHandler: AidlabSynchronizationDelegate {
    
    private var listener: AidlabFlutter.PlatformAidlabSynchronizationDelegate? = null
    
    fun init(binding: FlutterPlugin.FlutterPluginBinding) {
        listener = AidlabFlutter.PlatformAidlabSynchronizationDelegate(binding.binaryMessenger)
    }

    fun release() {
        listener = null
    }
    
    override fun didReceivePastActivity(aidlab: IAidlab, timestamp: Long, activity: ActivityType) {
        listener?.didReceivePastActivity(AidlabFlutter.ActivityEvent().apply {
            this.activityType = activity.value.toLong()
        }) {}
    }

    override fun didReceivePastECG(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        listener?.didReceivePastECG(AidlabFlutter.EcgEvent().apply {
            this.timestamp = timestamp
            this.values = DoubleArray(values.size) { index -> values[index].toDouble() }
        }) {}
    }

    override fun didReceivePastHeartRate(aidlab: IAidlab, timestamp: Long, hrv: IntArray, heartRate: Int) {
        listener?.didReceivePastHeartRate(AidlabFlutter.HeartRateEvent().apply {
            this.timestamp = timestamp
            this.hrv = hrv
            this.heartRate = heartRate.toLong()
        }) {}
    }

    override fun didReceivePastRespiration(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        listener?.didReceivePastRespiration(AidlabFlutter.RespirationEvent().apply {
            this.timestamp = timestamp
            this.values = DoubleArray(values.size) {index -> values[index].toDouble()}
        }) {}
    }

    override fun didReceivePastRespirationRate(aidlab: IAidlab, timestamp: Long, value: Int) {
        listener?.didReceivePastRespirationRate(AidlabFlutter.RespirationRateEvent().apply {
            this.timestamp = timestamp
            this.value = value.toLong()
        }) {}
    }

    override fun didReceivePastSkinTemperature(aidlab: IAidlab, timestamp: Long, value: Float) {
        listener?.didReceivePastSkinTemperature(AidlabFlutter.SkinTemperatureEvent().apply {
            this.timestamp = timestamp
            this.value = value.toDouble()
        }) {}
    }

    override fun didReceivePastSteps(aidlab: IAidlab, timestamp: Long, value: Long) {
        listener?.didReceivePastSteps(AidlabFlutter.StepsEvent().apply {
            this.timestamp = timestamp
            this.value = value
        }) {}
    }

    override fun didReceiveUnsynchronizedSize(aidlab: IAidlab, unsynchronizedSize: Int) {
        listener?.didReceiveUnsynchronizedSize(AidlabFlutter.UnsynchronizedSizeEvent().apply {
            this.unsynchronizedSize = unsynchronizedSize.toLong()
        }) {}
    }

    override fun syncStateDidChange(aidlab: IAidlab, state: SyncState) {
        Log.d("AIDLAB_FLUTTER", "syncStateDidChange $state")
        listener?.syncStateDidChange(AidlabFlutter.SyncStateEvent().apply {
            this.state = state.value.toLong()
        }) {}
    }
}