//
//  AidlabSynchronizationHandler.swift
//  AidlabSDK
//
//  Created by Szymon Gęsicki on 05/02/2021.
//  Copyright © 2020 Aidlab. All rights reserved.
//

import Flutter
import UIKit
import AidlabSDK

public class AidlabSynchronizationHandler: AidlabSynchronizationDelegate {
    
    private var listener: PlatformAidlabSynchronizationDelegate?
    
    func initial(binding: FlutterBinaryMessenger) {
        listener = PlatformAidlabSynchronizationDelegate(binaryMessenger: binding)
    }
    
    func release() {
        listener = nil
    }
    
    public func syncStateDidChange(_ aidlab: IAidlab, state: SyncState) {
        listener?.syncStateDidChange(SyncStateEvent().apply(closure: {
            $0.state = NSNumber(value: state.rawValue)
        }), completion: { _ in })
    }
    
    public func didReceivePastECG(_ aidlab: IAidlab, timestamp: UInt64, values: [Float]) {
        listener?.didReceivePastECG(EcgEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.values = FlutterStandardTypedData.init(float64: Data(values: values.map { Double($0) }))
        }), completion: { _ in })
    }
    
    public func didReceivePastRespiration(_ aidlab: IAidlab, timestamp: UInt64, values: [Float]) {
        listener?.didReceivePastRespiration(RespirationEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.values = FlutterStandardTypedData.init(float64: Data(values: values.map { Double($0) }))
        }), completion: { _ in })
    }
    
    public func didReceivePastSkinTemperature(_ aidlab: IAidlab, timestamp: UInt64, value: Float) {
        listener?.didReceivePastSkinTemperature(SkinTemperatureEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.value = NSNumber(value: value)
        }), completion: { _ in })
    }
    
    public func didReceivePastHeartRate(_ aidlab: IAidlab, timestamp: UInt64, hrv: [Int32], heartRate: Int32) {
        listener?.didReceivePastHeartRate(HeartRateEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.heartRate = NSNumber(value: heartRate)
            $0.hrv = FlutterStandardTypedData.init(int32: Data(values: hrv))
        }), completion: { _ in })
    }
    
    public func didReceiveUnsynchronizedSize(_ aidlab: IAidlab, unsynchronizedSize: UInt16) {
        listener?.didReceiveUnsynchronizedSize(UnsynchronizedSizeEvent().apply(closure: {
            $0.unsynchronizedSize = NSNumber(value: unsynchronizedSize)
        }), completion: { _ in })
    }
    
    public func didReceivePastRespirationRate(_ aidlab: IAidlab, timestamp: UInt64, value: UInt32) {
        listener?.didReceivePastRespirationRate(RespirationRateEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.value = NSNumber(value: value)
        }), completion: { _ in })
    }
    
    public func didReceivePastActivity(_ aidlab: IAidlab, timestamp: UInt64, activity: ActivityType) {
        listener?.didReceivePastActivity(ActivityEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.activityType = NSNumber(value: activity.rawValue)
        }), completion: { _ in })
    }
    
    public func didReceivePastSteps(_ aidlab: IAidlab, timestamp: UInt64, value: UInt64) {
        listener?.didReceivePastSteps(StepsEvent().apply(closure: {
            $0.timestamp = NSNumber(value: timestamp)
            $0.value = NSNumber(value: value)
        }), completion: { _ in })
    }
}
